local math = require("math")

-- Multiplier on the cost of belts
local beltCostFactor = 5
-- Multiplier on the speed of belts
local beltSpeedFactor = 1/3
-- Constant additional belt speed (if belts are too slow they won't work at all)
local beltSpeedMod = 0.01

-- Multiplier on the recipe cost of logistic robots
local botCostFactor =  2
-- Multiplier on the energy cost of logistic robots. Roboports are also changed to be capable of supplying proportionally more energy.
local botEnergyMultiplier = 5

-- Iterare over belts
local beltishnames = {}
for _, belt in pairs(data.raw["transport-belt"]) do
    beltishnames[belt.name] = true
    belt.speed = belt.speed * beltSpeedFactor + beltSpeedMod
end

-- Iterare over splitters
for _, belt in pairs(data.raw["transport-belt"]) do
    beltishnames[belt.name] = true
    belt.speed = belt.speed * beltSpeedFactor + beltSpeedMod
end

-- Iterare over underground belts
for _, belt in pairs(data.raw["splitter"]) do
    beltishnames[belt.name] = true
    belt.speed = belt.speed * beltSpeedFactor + beltSpeedMod
end

-- multiplyEnergy("100kW", 5) yields "500kW"
function multiplyEnergy(multiplicand, multiplier)
    local switchindex = string.find(multiplicand, "[%u%l]")
    local numberPart = string.sub(multiplicand, 1, switchindex-1)
    local unitpart = string.sub(multiplicand, switchindex)
    local newvalue = math.ceil(tonumber(numberPart) * multiplier)
    return tostring(newvalue) .. unitpart
end

-- Iterate over bots
local botnames = {}
for _, bot in pairs(data.raw["logistic-robot"]) do
    botnames[bot.name] = true -- Save the name so we can change the recipe
    bot["max_energy"] = multiplyEnergy(bot["max_energy"], botEnergyMultiplier)
    bot["energy_per_tick"] = multiplyEnergy(bot["energy_per_tick"], botEnergyMultiplier)
    bot["energy_per_move"] = multiplyEnergy(bot["energy_per_move"], botEnergyMultiplier)
end

-- Iterate over roboports
local roboportnames = {}
for _, roboport in pairs(data.raw.roboport) do
    roboportnames[roboport.name] = true -- Save the name so we can change the recipe
    roboport["recharge_minimum"] = multiplyEnergy(roboport["recharge_minimum"], botEnergyMultiplier)
    roboport["energy_usage"] = multiplyEnergy(roboport["energy_usage"], botEnergyMultiplier)
    roboport["charging_energy"] = multiplyEnergy(roboport["charging_energy"], botEnergyMultiplier)
    local source = roboport["energy_source"]
    source["input_flow_limit"] = multiplyEnergy(source["input_flow_limit"], botEnergyMultiplier)
    source["buffer_capacity"] = multiplyEnergy(source["buffer_capacity"], botEnergyMultiplier)
end

-- Make all belts more expensive
for name, recipe in pairs(data.raw.recipe) do
    if beltishnames[recipe.result] then
        for k, v in pairs(recipe.ingredients) do
            local flag = true
            for ik, iv in pairs(v) do
                if beltishnames[ik] or beltishnames[iv] then
                    flag = false
                end
            end
            if(flag) then
                for ik, iv in pairs(v) do
                    if type(iv) == "number" then
                        v[ik] = math.ceil(iv * beltCostFactor)
                    end
                end
            end
        end
    end
end

-- Make all robots more expensive
for name, recipe in pairs(data.raw.recipe) do
    if botnames[recipe.result] then
        for _, v in pairs(recipe.ingredients) do
            for ik, iv in pairs(v) do
                if type(iv) == "number" then
                    v[ik] = math.ceil(iv * botCostFactor)
                end
            end
        end
    end
end


--Technology tweaks to allow early-game trains
data.raw.technology["railway"].prerequisites = {"engine"}
data.raw.technology["railway"].unit.count = 5
data.raw.technology["railway"].unit.time = 5
data.raw.technology["railway"].unit.ingredients = {{"science-pack-1", 1}}

data.raw.technology["engine"].prerequisites = {}
data.raw.technology["engine"].unit.count = 5
data.raw.technology["engine"].unit.time = 5
data.raw.technology["engine"].unit.ingredients = {{"science-pack-1", 1}}

data.raw.technology["automated-rail-transportation"].prerequisites = {"railway"}
data.raw.technology["automated-rail-transportation"].unit.count = 10
data.raw.technology["automated-rail-transportation"].unit.time = 10
data.raw.technology["automated-rail-transportation"].unit.ingredients = {{"science-pack-1", 1}}

data.raw.technology["rail-signals"].prerequisites = {"railway"}
data.raw.technology["rail-signals"].unit.count = 10
data.raw.technology["rail-signals"].unit.time = 10
data.raw.technology["rail-signals"].unit.ingredients = {{"science-pack-1", 1}}
    
--recipe tweaks to allow early-game trains (and make them a bit cheaper)
data.raw.recipe["engine-unit"].ingredients = {{"iron-gear-wheel", 5}, {"pipe", 2}}
data.raw.recipe["engine-unit"].category = "crafting" --Makes them craftable by hand; otherwise they still require blue assemblers even having only two inputs
data.raw.recipe["diesel-locomotive"].ingredients = {{"engine-unit", 25}, {"electronic-circuit", 5}}
data.raw.recipe["cargo-wagon"].ingredients = {{"iron-gear-wheel", 10}, {"iron-plate", 50}}
data.raw.recipe["train-stop"].ingredients = {{"electronic-circuit", 5}, {"iron-plate", 20}}
data.raw.recipe["rail-signal"].ingredients = {{"electronic-circuit", 1}, {"iron-plate", 2}}
data.raw.recipe["straight-rail"].ingredients = {{"iron-stick", 10}, {"stone", 1}}
data.raw.recipe["straight-rail"]["energy_required"] = 2
data.raw.recipe["curved-rail"].ingredients = {{"iron-stick", 40}, {"stone", 4}}
data.raw.recipe["curved-rail"]["energy_required"] = 8
    